@@ -9,8 +9,7 @@ from duckduckgo_search import ddg
 from google_searching import ggl
 from google_trends import realtime_trends
 
-
-__version__ = '1.0'
+__version__ = '1.3'
 
 THREADS = 1
 MIN_WAIT = 1
@@ -22,12 +21,20 @@ BLACKLIST = ("https://t.co", "t.umblr.com", "messenger.com",
              "mediawiki", ".css", ".ico", ".xml", "intent/tweet",
              "twitter.com/share", "signup", "login", "dialog/feed?",
              ".json", ".svg", ".gif", "zendesk", "clickserve",
-             "mailto:", "smart-captcha/", "Login", "mail.google.com",
+             "mailto:", "smart-captcha/", "Login", "mail.",
              ".jpg", ".jpeg", ".png", ".iso", ".js", "s.click",
              "javascript:", "whatsapp://", "tel:", "tg://", "/#",
              "showcaptcha?", "/share.php?", "_click_", "/authorize?",
              "/join?", ".cs", "/joinchat", "/auth/", "t.me/share",
-             "Special:",)
+             "Special:", "/help", "support.", "/support", "/chat",
+             "/captcha", "policies", "/terms", "/privacy")
+HEADERS = {
+    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8",
+    "Accept-Encoding": "gzip, deflate, br",
+    "Upgrade-Insecure-Requests": "1",
+    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; rv:78.0) Gecko/20100101 Firefox/78.0",
+    }
+URLS_CACHE = set()
 
 def debug_print(*agrs, **kwargs):
     if DEBUG:
@@ -42,29 +49,38 @@ def real_trends(country='US', language='en-US', category='h'):
             print(f'Google trends error. Sleep 25-35 sec')
             sleep(uniform(25, 35))
 
-def get_url(url):
-    headers = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; rv:78.0) Gecko/20100101 Firefox/78.0"}
+def url_in_blacklist(url):
+    if any(x in url for x in BLACKLIST):
+        debug_print(f'{url}, STATUS: in BLACKLIST')
+        return True
+
+def url_fix(url):
     if 'https://' not in url and 'http://' not in url:
         url = f"https://{url}"
     try:
       url = url[:url.rindex('#')]
     except:
       pass
-    debug_print(f'{url}, STATUS: request')
-    try:
-        resp = requests.get(url, headers=headers, timeout=4)
-        if resp.status_code == 200:
-            debug_print(f'{resp.url}, STATUS: {resp.status_code}')
-            if any(x in resp.url for x in BLACKLIST):
-                debug_print(f'{resp.url}, STATUS: in BLACKLIST')
-                return None            
-            return resp
-        debug_print(resp.raise_for_status())      
-    except requests.ConnectionError:
-        debug_print(f'{url}, STATUS: Connection error. Sleep 25-35 sec')
-        sleep(uniform(25, 35))
-    except:
-        debug_print(f'{url}, STATUS: ERROR')
+    return url    
+
+def get_url(url):
+    url = url_fix(url)
+    if url not in URLS_CACHE and not url_in_blacklist(url):
+        debug_print(f'{url}, STATUS: request')
+        try:            
+            resp = requests.get(url, headers=HEADERS, timeout=4)
+            URLS_CACHE.add(url)
+            if resp.status_code == 200:
+                debug_print(f'{resp.url}, STATUS: {resp.status_code}')
+                if url_in_blacklist(resp.url):
+                    return None            
+                return resp
+            debug_print(resp.raise_for_status())      
+        except requests.ConnectionError:
+            debug_print(f'{url}, STATUS: Connection error. Sleep 25-35 sec')
+            sleep(uniform(25, 35))
+        except:
+            debug_print(f'{url}, STATUS: ERROR')
 
 def google_search(word, max_results=20):
     query  = word.replace(' ','+')
@@ -75,6 +91,7 @@ def google_search(word, max_results=20):
 def ddg_search(word, max_results=20):
     query  = word.replace(' ','+')
     search_ddg = ddg(query, max_results=max_results)
+    search_ddg = search_ddg[:max_results]
     urls = [x['href'].lstrip('https://') for x in search_ddg]
     return urls
 
@@ -126,8 +143,7 @@ def _thread(trend):
     
 def fake_traffic(country='US', language='en-US', category='h', threads=THREADS, min_wait=MIN_WAIT, max_wait=MAX_WAIT, debug=DEBUG):
     """
-    Imitating an Internet user by mimicking popular web traffic (internet traffic generator).
-    
+    Imitating an Internet user by mimicking popular web traffic (internet traffic generator).    
     country = country code ISO 3166-1 Alpha-2 code (https://www.iso.org/obp/ui/),
     language = country-language code ISO-639 and ISO-3166 (https://www.fincher.org/Utilities/CountryLanguageList.shtml),
     category = сategory of interest of a user (defaults to 'h'):
@@ -158,8 +174,8 @@ def fake_traffic(country='US', language='en-US', category='h', threads=THREADS,
             for i, trend in enumerate(trends, start=1):
                 print(f"Thread {i} start.")
                 executor.submit(_thread, trend)
-                sleep(uniform(25, 35))      
+                sleep(uniform(25, 35))
+        URLS_CACHE.clear()
 
 if __name__ == '__main__':
     fake_traffic(country='US', language='en-US')
-
